==> localnews-activity-service-3.4.5-sources.jar: com/bxm/localnews/activity/vote/strategy/impl/DayVoteStrategy.java <==
package com.bxm.localnews.activity.vote.strategy.impl;

import com.bxm.localnews.activity.dto.VoteDetailDTO;
import com.bxm.localnews.activity.param.VotePinParam;
import com.bxm.newidea.component.redis.KeyGenerator;
import com.bxm.newidea.component.tools.DateUtils;
import com.bxm.newidea.component.tools.RandomUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.Map;

import static com.bxm.localnews.common.constant.RedisConfig.USER_VOTE_KEY;

/**
 * 每日重置投票次数的投票策略
 * 每日可进行投票，直到投票结束，每日最多可投max条
 * @author liujia
 */
@Component
@Slf4j
public class DayVoteStrategy extends AbstractVoteStrategy {

    @Override
    KeyGenerator buildKey(Long userId, Long voteId) {
        String subKey = DateUtils.PATTERN_NO_DELIMITER_FORMAT.get().format(new Date());
        return USER_VOTE_KEY.copy().appendKey(voteId).appendKey(userId).appendKey(subKey);
    }

    @Override
    void expired(KeyGenerator key, Date endTime) {
        redisHashMapAdapter.expire(key, DateUtils.getCurSeconds() + RandomUtils.nextLong(1000, 100000));
    }

    @Override
    public void deal(VoteDetailDTO detail, VotePinParam param) {
        KeyGenerator key = buildKey(param.getUserId(), param.getVoteId());
        Map<String, Long> userVoteMap = redisHashMapAdapter.entries(key, Long.class);
        Long voteTotal = userVoteMap.get(VOTE_TOTAL);
        voteTotal = null == voteTotal ? 0L : voteTotal;
        Long times = userVoteMap.get(TIMES);

        //初始有一票
        long max = detail.getMax() + 1;

        //投票活动已过期
        if (detail.getExpired()) {
            setExpired(detail);
        } else if (null == times || times > 0) {
            //有投票次数，可进行投票
            setVote(detail);
        } else if (max > voteTotal) {
            //今日还可通过分享获得投票机会
            long haveTimes = max - voteTotal;
            detail.setStatus(ButtonStatus.ENABLE.name());
            detail.setStatusLabel("你已投票，点击分享可再投一次哦");
            detail.setSubTitle("今日还可再投" + haveTimes + "次");
            detail.setAction(Action.SHARE.name());
        } else if (voteTotal >= max) {
            detail.setStatus(ButtonStatus.DISABLE.name());
            detail.setStatusLabel("你已投票");
            detail.setSubTitle("今日投票次数已用完，明日可继续投哦");
            detail.setAction(Action.SHARE.name());
        }
    }

    @Override
    public String name() {
        return "DAY";
    }
}

==> localnews-activity-service-3.11.0-sources.jar: com/bxm/localnews/activity/vote/strategy/impl/DayVoteStrategy.java <==
package com.bxm.localnews.activity.vote.strategy.impl;

import com.bxm.localnews.activity.dto.VoteDetailDTO;
import com.bxm.localnews.activity.param.VotePinParam;
import com.bxm.newidea.component.redis.KeyGenerator;
import com.bxm.newidea.component.tools.DateUtils;
import com.bxm.newidea.component.tools.RandomUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.Map;

import static com.bxm.localnews.common.constant.RedisConfig.USER_VOTE_KEY;

/**
 * 每日重置投票次数的投票策略
 * 每日可进行投票，直到投票结束，每日最多可投max条
 *
 * @author liujia
 */
@Component
@Slf4j
public class DayVoteStrategy extends AbstractVoteStrategy {

    @Override
    KeyGenerator buildKey(Long userId, Long voteId) {
        String subKey = DateUtils.PATTERN_NO_DELIMITER_FORMAT.get().format(new Date());
        return USER_VOTE_KEY.copy().appendKey(voteId).appendKey(userId).appendKey(subKey);
    }

    @Override
    void expired(KeyGenerator key, Date endTime) {
        redisHashMapAdapter.expire(key, DateUtils.getCurSeconds() + RandomUtils.nextLong(1000, 100000));
    }

    @Override
    public void deal(VoteDetailDTO detail, VotePinParam param) {
        KeyGenerator key = buildKey(param.getUserId(), param.getVoteId());
        Map<String, Long> userVoteMap = redisHashMapAdapter.entries(key, Long.class);
        Long voteTotal = userVoteMap.get(VOTE_TOTAL);
        voteTotal = null == voteTotal ? 0L : voteTotal;
        Long times = userVoteMap.get(TIMES);

        //初始有一票
        long max = detail.getMax() + 1;

        //投票活动已过期
        if (detail.getExpired()) {
            setExpired(detail);
        } else if (detail.getNotStart()) {
            setNotStart(detail);
        } else if (null == times || times > 0) {
            //有投票次数，可进行投票
            setVote(detail);
        } else if (max > voteTotal) {
            //今日还可通过分享获得投票机会
            long haveTimes = max - voteTotal;
            detail.setStatus(ButtonStatus.ENABLE.name());
            detail.setStatusLabel("你已投票，点击分享可再投一次哦");
            detail.setSubTitle("今日还可再投" + haveTimes + "次");
            detail.setAction(Action.SHARE.name());
        } else if (voteTotal == max) {
            detail.setStatus(ButtonStatus.DISABLE.name());
            detail.setStatusLabel("你已投票");
            detail.setSubTitle("今日投票次数已用完，明日可继续投哦");
            detail.setAction(Action.SHARE.name());
        }
    }

    @Override
    public String name() {
        return "DAY";
    }
}
